#ifndef ACCOUNTPANELDELEGATE_H
#define ACCOUNTPANELDELEGATE_H
////////////////////////////////////////////////////////////////////////////////
/// AccountPanelDelegate.h
///
/// Copyright (c) 2006 MotiMotion
/// All rights reserved.
///
/// Author: Michael Rutt (michael@motimotion.com)
///		   http://blogs.motimotion.com/michael
////////////////////////////////////////////////////////////////////////////////

#import "AccountsController.h"

#import <Cocoa/Cocoa.h>


static const int DetailsPanelTypeTag				= 1;
static const int DetailsPanelBinderTag				= 2;
static const int DetailsPanelAccountTag				= 3;
static const int DetailsPanelBudgetTag				= 4;
static const int DetailsPanelReportTag				= 5;

static const int BinderDetailsNameTag				= 100;
static const int BinderDetailsIconTag				= 101;
static const int BinderDetailsExcludeBalanceTag		= 102;
static const int BinderDetailsDoNotShowTotalTag		= 103;

static const int AccountDetailsNameTag				= 201;
static const int AccountDetailsTypeTag				= 202;
static const int AccountDetailsBalanceTag			= 203;
static const int AccountDetailsIconTag				= 204;
static const int AccountDetailsBalanceLabelTag		= 205;
static const int AccountDetailsExcludeBalanceTag	= 206;
static const int AccountDetailsWebsiteTag			= 207;
static const int AccountDetailsCreditLimitLabelTag	= 221;
static const int AccountDetailsCreditLimitTag		= 222;


static const int BudgetDetailsNameTag				= 300;
static const int BudgetDetailsActiveTag				= 301;

static const int ReportDetailsNameTag				= 400;
static const int ReportDetailsTypeTag				= 401;

////////////////////////////////////////////////////////////////////////////////
/// The AccountPanelDelegate handles all actions from the account panel that
/// is displayed when the user is adding a new account or binder.  It also
/// handles displaying the account panel as a sheet attached to a specified
/// window.
////////////////////////////////////////////////////////////////////////////////
@interface AccountPanelDelegate : NSObject
{
	/// The accounts controller that manages the list of accounts and binders
	IBOutlet AccountsController*	accountsController;
	
	/// The binder selection controller that manages the list of binders that
	/// can be selected when creating a new account
	IBOutlet id						binderSelectionController;
	
	/// The binder details view.
	IBOutlet NSView*				binderView;
	
	/// The account details view.
	IBOutlet NSView*				accountView;
	
	/// The budget details view.
	IBOutlet NSView*				budgetView;
	
	/// The report details view.
	IBOutlet NSView*				reportView;
	
	IBOutlet NSOutlineView*			accountOutlineView;
	
	/// The binder description field.
    IBOutlet NSTextView*			binderDescriptionField;
	
	/// The budget description field.
    IBOutlet NSTextView*			budgetDescriptionField;
	
	/// The NSBox used to display the currently selected details.
	IBOutlet NSBox*					detailsBox;

	/// The account panel to be displayed as a sheet
	IBOutlet NSPanel*				accountPanel;
	
	// The window that owns the managed context and will display the sheet
	IBOutlet id						window;
	
	/// The current account being edited; this is avoided in the duplicate
	/// name check
	NSManagedObject*				_currentAccount;
	
	/// The destination binder for the account being created
	NSManagedObject*				_destinationBinder;
}

////////////////////////////////////////////////////////////////////////////////
/// Shows the account panel as a sheet attached to the window outlet.  The sheet
/// opens in new mode creating a new account based upon the currently selected
/// account.
////////////////////////////////////////////////////////////////////////////////
- (void) showSheet;

////////////////////////////////////////////////////////////////////////////////
/// Shows the account panel as a sheet attached to the window outlet.  The sheet
/// opens in edit mode for the currently selected account.
////////////////////////////////////////////////////////////////////////////////
- (void) showSheetForEditing;

////////////////////////////////////////////////////////////////////////////////
/// Called when the user changes the type of account to create.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) accountTypeChanged: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called when the user changes the sub-type of account to create (i.e.
/// checking, savings, etc).
////////////////////////////////////////////////////////////////////////////////
- (IBAction) accountSubtypeChanged: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called when the Create button has been pressed on the account sheet.  This
/// function determine which tab is currently visible (Account or Binder) and
/// calls the appropriate function to create the new account using that data.
/// If the fields are all valid, the new account or binder is created and the
/// sheet is closed.  If a problem occurs, a message is displayed to the user
/// and the sheet remains open.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) sheetOK: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called when the Cancel button has been pressed on the account sheet.  Closes
/// the sheet and returns a cancel action.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) sheetCancel: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called automatically whenever the account sheet closes.  Clears all fields on
/// the sheet to ensure a clean display the next time it is shown.
////////////////////////////////////////////////////////////////////////////////
- (void) sheetDidEndForNew: (NSWindow*) sheet
				returnCode: (int) returnCode
			   contextInfo: (void*) contextInfo;

- (void) sheetDidEndForEditing: (NSWindow*) sheet
			    	returnCode: (int) returnCode
			       contextInfo: (void*) contextInfo;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) updateBinder;
- (void) updateBinder: (NSManagedObject*) binder;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) updateBankAccount;
- (void) updateBankAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) updateBudget;
- (void) updateBudget: (NSManagedObject*) budget;

////////////////////////////////////////////////////////////////////////////////
/// Creates a new binder based on the field values on the account panel.
////////////////////////////////////////////////////////////////////////////////
- (void) createBinder;

////////////////////////////////////////////////////////////////////////////////
/// Creates a new account based on the field values on the account panel.
////////////////////////////////////////////////////////////////////////////////
- (void) createAccount;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) createBudget;



- (IBAction) duplicateBudget: (id) sender;

- (IBAction) makeBudgetActive: (id) sender;

- (void) createReport;

- (void) updateReport;

- (void) updateReport: (NSManagedObject*) report;

////////////////////////////////////////////////////////////////////////////////
/// Calls one of the other validate...Fields() methods depending upon the type
/// of account being created.  Returns YES if all fields are valid.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) validatePanelFields;

////////////////////////////////////////////////////////////////////////////////
/// Validates the information on the account panel.  Returns YES if all data
/// is acceptable.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) validateAccountFields;

////////////////////////////////////////////////////////////////////////////////
/// Validates the information on the binder panel.  Returns YES if all data
/// is acceptable.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) validateBinderFields;

////////////////////////////////////////////////////////////////////////////////
/// Validates the information on the budget panel.  Returns YES if all data
/// is acceptable.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) validateBudgetFields;

- (BOOL) validateReportFields;

@end

#endif