////////////////////////////////////////////////////////////////////////////////
//  AccountTableView.h
////////////////////////////////////////////////////////////////////////////////
///
/// @brief	NSTableView class that displays the transactions for a bank account.
///
/// This class displays transactions for the selected bank account.
///
/// Copyright (c) 2006, 2007 MotiMotion.
///
/// @author	Michael Rutt
///			<a href="mailto:michael@motimotion.com">michael@motimotion.com</a>
///			<a href="http://www.mikerutt.com">http://www.mikerutt.com</a>
///
////////////////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>
#import <MotiFoundation/MotiTableView.h>


@interface AccountTableView : MotiTableView
{
	/// The controller containing the accounts
	IBOutlet id			accountsController;
	
	IBOutlet id			categorySelectionController;
	
	/// The NSMenu containing the columns to be selected
	IBOutlet id			columnsMenu;
	
	/// The split transaction window
	IBOutlet id			splitWindow;
	
	IBOutlet id			splitsController;
	
	/// The payee controller
	IBOutlet id			payeeController;

	// Completed was called
	BOOL					completedCalled;
	
	// Previous size of the string in the text view
	int						prevSize;
	
	// Was textDidChange called within itself due to a call to complete?
	int						inTextDidChange;
	
	/// Set to YES if the date has been changed in the row during an edit
	BOOL					dateChanged;
	
	/// Set to true if the user changes the payee from a nil value
	BOOL					_payeeChangedFromNil;
	
	/// The last row edited (used to detect when tab or mouse clicks change
	/// a row
	int						lastRowEdited;
	
	/// Stores the columns that are currently hidden in the table; the key is
	/// the column itself and the value is the index where the column was
	/// last positioned
	NSMutableDictionary*	hiddenColumns;

	/// The color to use for the balance column if set to color it
	NSColor*				_balanceColColor;
}

////////////////////////////////////////////////////////////////////////////////
/// Toggles the visibility of a column in the table.  The sender must be an
/// NSMenuItem with an NSOn/OffState assigned to it.  This state is used to
/// determine whether or not to hide the column.  (e.g. if it is checked, the
/// column is visible and will be hidden when this is called.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) toggleColumnItem: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Sets a column with the specified identifier as hidden (YES) or unhidden (NO)
/// and moves it to the hiddenColumns list or moves it out of the list and 
/// inserts it into the table.
////////////////////////////////////////////////////////////////////////////////
- (void) setColumnAsHidden: (BOOL) hidden
			withIdentifier: (NSString*) identifier;

////////////////////////////////////////////////////////////////////////////////
/// Moves a column with the specified identifier to the requested index.  If
/// the column is hidden (i.e. in the hiddenColumns dictionary), it will be
/// updated to indicate its last index so if it is unhidden it will be inserted
/// properly into the visible table columns.
////////////////////////////////////////////////////////////////////////////////
- (void) moveColumnWithIdentifier: (NSString*) identifier
						  toIndex: (int) index;

////////////////////////////////////////////////////////////////////////////////
/// Sets the width of the specified column.  If the column is in the
/// hiddenColumns list it will be set to when it is unhidden it is the correct
/// width.
////////////////////////////////////////////////////////////////////////////////
- (void) setWidth: (float) width
	ofColumnWithIdentifier: (NSString*) identifier;

/*
////////////////////////////////////////////////////////////////////////////////
/// Returns YES if the [self editedColumn] is the last editable column
/// in the row.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) lastEditableColumn;

////////////////////////////////////////////////////////////////////////////////
/// Returns YES if the column at position index is the last editable column
/// in the row.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) lastEditableColumn: (int) index;
*/

////////////////////////////////////////////////////////////////////////////////
/// 
////////////////////////////////////////////////////////////////////////////////
- (IBAction) jumpToTransferForSelectedItem: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// 
////////////////////////////////////////////////////////////////////////////////
- (IBAction) showSplitForSelectedItem: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// 
////////////////////////////////////////////////////////////////////////////////
- (IBAction) createSplitForSelectedItem: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Returns an array of the hidden NSTableColumns.
////////////////////////////////////////////////////////////////////////////////
- (NSDictionary*) hiddenColumns;

@end
