//
//  BudgetManager.h
//  Prospects
//
//  Created by Michael Rutt on 5/21/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// Months are numbered 1 - 12, so position 0 is set to 0 as a placeholder
static const int daysPerMonth[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
// Indexes of data in the array that is the value of the monthlyIncomeExpense dictionary
static const int BMIncome		= 0;
static const int BMExpense		= 1;
static const int BMSavings		= 2;
static const int BMCumIncome	= 3;
static const int BMCumExpense	= 4;
static const int BMCumSavings	= 5;
// Indexes of data in the array that is the value of the cateogories dictionary
static const int BMType 		= 0;
static const int BMValue		= 1;
static const int BMCumValue		= 2;

@interface BudgetManager : NSObject
{
	/// Outlet to the budget item controller
	IBOutlet id				budgetItemController;
	
	/// Array of NSCalendarDates of the month/years covered by the budget
	/// manager in ascending order
	NSMutableArray*			months;
	
	/// Dictionary of income/expenses:
	///    key = string representation of calendar month abbreviation & 2-digit year
	///				(i.e. "Jan 07" = [date descriptionWithCalendarFormat: @"%b %Y"] )
	///    value = NSArray of values for month
	///				(income, expense, savings, cumulative income, cumulative expense, cumulative savings)
	NSMutableDictionary*	monthlyIncomeExpense;
	
	/// Dictionary of categories to categoryData dictionaries:
	///	   key = category NSManagedObjected
	///  value = dictionary with data:
	///       key = string representation of calendar month abbreviation & 2-digit year
	///		   		  (i.e. "Jan 07" = [date descriptionWithCalendarFormat: @"%b %Y"] )
	///       value = NSArray of values for category for the month
	///				  (type of item (BudgetItemType), income/expense/savings, cumulative income/expense/savings)
	NSMutableDictionary*	categories;
	
	/// Automatically refresh when content changes; defaults to NO
	BOOL					_autoRefresh;
	
	/// Month/year to calculate data from and the number of months
	int						_month;
	int						_year;
	int						_numMonths;
}

/// Defines the possible types of budget data
typedef enum
{
	BudgetIncomeType	= 0,
	BudgetExpenseType	= 1,
	BudgetSavingsType	= 2
} BudgetItemType;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) resetData;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) refreshData;

////////////////////////////////////////////////////////////////////////////////
/// Returns an array of NSCalendarDates available in the budget manager.
////////////////////////////////////////////////////////////////////////////////
//- (NSArray*) 

////////////////////////////////////////////////////////////////////////////////
/// Calculates all the data based upon the specified range.
////////////////////////////////////////////////////////////////////////////////
- (void) calculateDataFromMonth: (int) month
						   year: (int) year
  			  forNumberOfMonths: (int) numMonths;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) incomeForMonth: (int) month year: (int) year;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) expenseForMonth: (int) month year: (int) year;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) savingsForMonth: (int) month year: (int) year;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) incomeForMonth: (int) month
						year: (int) year
					category: (NSManagedObject*) category
				   cumulative: (BOOL) cumulative;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) expenseForMonth: (int) month
						 year: (int) year
					 category: (NSManagedObject*) category
				   cumulative: (BOOL) cumulative;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) savingsForMonth: (int) month
						 year: (int) year
					 category: (NSManagedObject*) category
				   cumulative: (BOOL) cumulative;
				   
- (NSNumber*) cumulativeYTDIncomeForCategory: (NSManagedObject*) category;

- (NSNumber*) cumulativeYTDExpenseForCategory: (NSManagedObject*) category;

- (NSNumber*) cumulativeYTDSavingsForCategory: (NSManagedObject*) category;
				   
////////////////////////////////////////////////////////////////////////////////
/// Returns the surplus for a given month/year.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) surplusForMonth: (int) month year: (int) year;

////////////////////////////////////////////////////////////////////////////////
/// Returns the YTD cumulativesurplus.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) cumulativeYTDSurplus;

////////////////////////////////////////////////////////////////////////////////
/// Returns the cumulative surplus for a given month/year from with the
/// starting month / year provided in the CalculateData... call.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) cumulativeSurplusForMonth: (int) month year: (int) year;

////////////////////////////////////////////////////////////////////////////////
/// Returns the cumulative savings for a given month/year from with the
/// starting month / year provided in the CalculateData... call.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) cumulativeSavingsForMonth: (int) month year: (int) year;


////////////////////////////////////////////////////////////////////////////////
/// Returns the cumulative surplus for a given month/year from with the
/// starting month / year provided in the CalculateData... call.
////////////////////////////////////////////////////////////////////////////////
//- (NSNumber*) cumulativeSurplusForMonth: (int) month year: (int) year
//					  startingWithMonth: (int) startMonth year: (int) startYear;

////////////////////////////////////////////////////////////////////////////////
/// Returns the cumulative savings for a given month/year from with the
/// starting month / year provided in the CalculateData... call.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) cumulativeSavingsForMonth: (int) month year: (int) year;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageSurplus;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageIncomeForCategory: (NSManagedObject*) category;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageExpenseForCategory: (NSManagedObject*) category;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageSavingsForCategory: (NSManagedObject*) category;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageYTDSurplus;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageYTDIncomeForCategory: (NSManagedObject*) category;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageYTDExpenseForCategory: (NSManagedObject*) category;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageYTDSavingsForCategory: (NSManagedObject*) category;

////////////////////////////////////////////////////////////////////////////////
/// Returns the average income amount for a specific category during the time
/// from the specified starting month / year to the last month / year maintained
/// by the manager.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageIncomeForCategory: (NSManagedObject*) category
						 startingMonth: (int) month
								  year: (int) year;

////////////////////////////////////////////////////////////////////////////////
/// Returns the average expense amount for a specific category during the time
/// from the specified starting month / year to the last month / year maintained
/// by the manager.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageExpenseForCategory: (NSManagedObject*) category
						  startingMonth: (int) month
								   year: (int) year;

////////////////////////////////////////////////////////////////////////////////
/// Returns the average savings amount for a specific category during the time
/// from the specified starting month / year to the last month / year maintained
/// by the manager.
////////////////////////////////////////////////////////////////////////////////
- (NSNumber*) averageSavingsForCategory: (NSManagedObject*) category
						  startingMonth: (int) month
								   year: (int) year;

////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
- (void) setAutoRefresh: (BOOL) value;

@end
