////////////////////////////////////////////////////////////////////////////////
//  CategoryViewPointController.h
////////////////////////////////////////////////////////////////////////////////
///
/// @brief	Controls a Category ViewPoint instance.
///
/// This class controls a category viewpoint instance.  It is responsible for
/// updating the pie chart when:
///   - the selected account changes
///   - category name changes
///   - a transaction's category changes
///   - the account balance changes.
///
/// Copyright (c) 2006, 2007 MotiMotion.
///
/// @author	Michael Rutt
///			<a href="mailto:michael@motimotion.com">michael@motimotion.com</a>
///			<a href="http://www.mikerutt.com">http://www.mikerutt.com</a>
///
////////////////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>

#import "ViewPointComponentController.h"


////////////////////////////////////////////////////////////////////////////////
/// Controls a Category ViewPoint.
////////////////////////////////////////////////////////////////////////////////
@interface CategoryViewPointController : ViewPointComponentController
{
	/// The categories controller
	IBOutlet id					categoriesController;
	
	/// If YES, uses the debit value for computing the viewpoint; if NO uses
	/// the deposit value.
	BOOL						useDebitValue;
	
	/// Name of this controller
	NSString*					controllerName;
	
	/// Set to YES once category registrations have taken place
	BOOL						registeredCategoryObservations;
	
	// Minimum date to look for when filtering on transactions
	NSCalendarDate*				startingDate;
	
	/// Object to sync on
	NSString*					sync;
	
	/// Set to YES when changing accounts; otherwise NO; this is used to
	/// determine if we should delay after being made active or not (only delay
	/// when changing accounts)
	BOOL						_justChangedAccounts;
}

/// Number of seconds to delay before updating the viewpoint via a thread
static const float DISPLAY_DELAY = 0.5;

static const int FILTER_TAG			= 2;
static const int TIMEFRAME_TEXT_TAG	= 3;



//==============================================================================
// Class methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Called whenever the user changes the type of category breakdown to display,
/// which can be Expenses or Income.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) categoryTypeChanged: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called whenever the user drills down to a lower sub-category. 
////////////////////////////////////////////////////////////////////////////////
- (IBAction) topLevelCategoryChanged: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// When drilled down, can be called to move back up a level in the category
/// levels.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) moveUpCategoryLevel: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called when the slider containing the timeframe to view for the category
/// breakdown is changed.
////////////////////////////////////////////////////////////////////////////////
- (void) timeframeChanged;

////////////////////////////////////////////////////////////////////////////////
/// Registers for necessary observations with each category in order to track
/// changes to their names and transactions associated with them.
////////////////////////////////////////////////////////////////////////////////
- (void) registerObservationsForCategories;

////////////////////////////////////////////////////////////////////////////////
/// Unregisters all observations registered in registerObservationsForCategories.
////////////////////////////////////////////////////////////////////////////////
- (void) unregisterObservationsForCategories;

////////////////////////////////////////////////////////////////////////////////
/// Updates a category label on the chart when the name changes.
////////////////////////////////////////////////////////////////////////////////
- (void) updateCategory: (NSManagedObject*) category
		withOldLongName: (NSString*) oldLongName;

////////////////////////////////////////////////////////////////////////////////
/// Refreshes the displayed data; typically called if the type of data (debit or
/// deposit) changes.
////////////////////////////////////////////////////////////////////////////////
- (void) refreshData: (id) argument;

////////////////////////////////////////////////////////////////////////////////
/// Called after the view has been made active.
////////////////////////////////////////////////////////////////////////////////
- (void) viewMadeActive;

////////////////////////////////////////////////////////////////////////////////
/// Sets the timeframe label on the viewpoint based upon the current slider
/// position.
////////////////////////////////////////////////////////////////////////////////
- (void) setTimeframeLabel;

////////////////////////////////////////////////////////////////////////////////
/// Computes the startingDate based upon the timeframe.
////////////////////////////////////////////////////////////////////////////////
- (void) computeTimeframe;


//==============================================================================
// Overridden methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Overrides base class.
////////////////////////////////////////////////////////////////////////////////
- (void) updateViewPointForAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// Overrides base class.
////////////////////////////////////////////////////////////////////////////////
- (void) registerObservationsForAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// Overrides base class.
////////////////////////////////////////////////////////////////////////////////
- (void) unregisterObservationsForAccount: (NSManagedObject*) account;

@end
