////////////////////////////////////////////////////////////////////////////////
//  ImportManager.h
////////////////////////////////////////////////////////////////////////////////
///
/// @brief	Prospects import manager.
///
/// The ImportManager class handles importing of any external financial document
/// files, including QIF and QFX.  It manages display of all the views and panels
/// used in the import process and acts as the delegate to any of these displays.
/// It also translates the imported data to Prospects' data model.
///
/// Copyright (c) 2006 MotiMotion.
///
/// @author	Michael Rutt
///			<a href="mailto:michael@motimotion.com">michael@motimotion.com</a>
///			<a href="http://www.mikerutt.com">http://www.mikerutt.com</a>
///
////////////////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>
#import <MotiFoundation/MotiImportExportManager.h>
#import "BaseTranslator.h"
#import "TransactionsController.h"
#import "Transaction.h"
#import "BankAccount.h"


typedef enum
{
	UNKNOWN_FILE_TYPE	= 0,
	QIF					= 1,
	OFX					= 2
} PFileIOType;

@interface ImportManager : MotiImportExportManager
{	
	/// The view to attach to the import panel containing Prospects specific
	/// options for the import
	IBOutlet id			importAccessoryView;
	
	/// The status panel displayed to the user during an import
	IBOutlet id			importStatusPanel;
	
	/// The NSBox used to display the import status on the importStatusPanel
	IBOutlet id			importStatusBox;
	
	/// The Prospects document used to access the data model
	IBOutlet id			document;
	
	/// The progress bar displayed during the import
	IBOutlet id			importProgressBar;
	
	/// The controller providing a selectable list of accounts
	IBOutlet id			accountSelectionController;
	
	/// Transaction Controller for adding new transactions
	IBOutlet TransactionsController*	transactionController;
	
	IBOutlet id			allTransactionsController;
	
	// Accounts Controller used to default the account selection in the 
	// import panel to the currently selected account
	IBOutlet id			accountsController;
	
	/// Status view displayed while reading in the file and subsequently asking
	/// the user for the account selection
	IBOutlet id			startImportView;

	IBOutlet id			ongoingStatusView;
	IBOutlet id			completedView;
	IBOutlet id			canceledView;
	IBOutlet id			confirmationView;
	
	/// The spinner used to indicate that rules are being applied to
	/// existing transactions
	IBOutlet id			applyRulesProgressIndicator;
	
	IBOutlet id			existingTransactionImportBox;
	IBOutlet id			confirmTransactionImportBox;

	/// Original frame of the panel without the confirmation box in it
	NSRect				importStatusPanelFrame;
		
	/// Is an import in progress?
	BOOL				importInProgress;
	
	/// How are we confirming the transactions?
	int					confirmationType;
	
	/// The import rules panel
	IBOutlet id			importRulesPanel;

	/// The import rules to use; if nil, no rules are to be applied
	IBOutlet id			importRulesController;
	
	/// The payee controller used to memorize transactions
	IBOutlet id			payeeController;
	
	/// Whether or not to apply import rules
	BOOL				applyImportRules;
	
	/// Whether or not to create import rules on the fly
	BOOL				createImportRules;
	
	/// Tracks number of skipped objects
	int					numberOfSkippedObjects;
	
	/// Tracks the number of duplicates not imported
	int					numberOfDuplicateObjects;
	
	/// The QIF translator (make this generic!)
	BaseTranslator*		translator;
	
	NSMutableArray*		transactionsNeedingConfirmation;
	
	/// A dictionary of mutable arrays; the key is the account name for the
	/// transfer and the sub-arrays contain a
	/// BankAccountTransaction that has been imported that is a transfer and
	/// the name of the account it is being transferred to; as transactions are
	/// imported that are transfers, it checks this array and if the match is not
	/// present, it adds itself to the array.  When a match is made, it will
	/// setup the transfer link and then remove the transaction from this array;
	/// once everything has been imported, the remaining transfers are created by
	/// Prospects (this may mean creating the account as well)
	NSMutableArray*		_transfers;
	
	/// Modal ID for the import panel when running
	int					_importPanelModalID;
	
	/// Index into transactionsNeedingConfirmation currently being processed
	int					_confirmationIndex;
	
	/// This is a transaction created temporarily during import that is used
	/// to check for duplicate transactions in the _importObject method; it
	/// is destroyed in finalizeImport
	NSManagedObject*	_dupeTransaction;
	
	// Account we are importing into
	BankAccount*		_importAccount;
	
	// Name of the account we're importing into
	NSString*			_importAccountName;
	
	NSArray*			_currentAccountTransactions;
}

static const int CONFIRM_POTENTIAL_DUPES_ONLY	= 0;
static const int CONFIRM_ALL_DUPES				= 1;
static const int CONFIRM_ALL_IMPORTS			= 2;

static const int IMPORT_STATUS_TAG				= 50;
static const int ACCOUNT_LABEL_TAG				= 51;
static const int ACCOUNT_SELECTION_POPUP_TAG	= 52;

static const int BEGIN_IMPORT_BUTTON_TAG		= 200;
static const int CANCEL_IMPORT_BUTTON_TAG		= 201;

static const int DATE_FORMAT_LABEL_TAG			= 300;
static const int DATE_FORMAT_SELECTION_TAG		= 301;
static const int DATE_FORMAT_EXAMPLE_TAG		= 302;
static const int DECIMAL_LABEL_TAG				= 303;
static const int DECIMAL_SELECTION_TAG			= 304;
static const int DECIMAL_EXAMPLE_TAG			= 305;

static const int COUNTER_TEXTFIELD_TAG			= 2000;
static const int SKIP_BUTTON_TAG				= 2001;
static const int IMPORT_BUTTON_TAG				= 2002;
static const int SKIP_REST_BUTTON_TAG			= 2003;
static const int IMPORT_REST_BUTTON_TAG			= 2004;

static const int CONFIRM_TRANS_DETAILS_TAG		= 3000;
static const int CONFIRM_TRANS_NOTES_TAG		= 3001;

/// Used to fill in type for
static const int POTENTIAL_DUPLICATE			= 0;
static const int ACTUAL_DUPLICATE				= 1;

////////////////////////////////////////////////////////////////////////////////
/// Action that displays the importPanel as a sheet.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) showImportSheet: (id)sender;

////////////////////////////////////////////////////////////////////////////////
/// Action that displays the importRulesPanel as a sheet.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) showImportRules: (id)sender;

////////////////////////////////////////////////////////////////////////////////
/// Action that cancels the import in progress.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) importPanelCancel: (id)sender;

////////////////////////////////////////////////////////////////////////////////
/// Applies the rules to the current list of transactions of the selected
/// account (i.e. what's in the transactionsController).
////////////////////////////////////////////////////////////////////////////////
- (IBAction) applyRulesToCurrentTransactions: (id) sender;

- (IBAction) requestAccountSelection: (id) sender;

- (IBAction) beginImport: (id) sender;

- (IBAction) closeImportPanel: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called when the user clicks the import button while determining if he/she
/// should import a dupe / potential dupe
////////////////////////////////////////////////////////////////////////////////
- (IBAction) importCurrentTransaction: (id) sender;

- (IBAction) importRemainderOfTransactions: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Called when the user clicks the skip button while determining if he/she
/// should import a dupe / potential dupe
////////////////////////////////////////////////////////////////////////////////
- (IBAction) skipCurrentTransaction: (id) sender;

- (IBAction) skipRemainderOfTransactions: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Delegate of the importPanel sheet that is called when the importPanel
/// closes.  This method runs through the process for importing an external
/// financial file based upon the selections made on the importAccessoryView.
////////////////////////////////////////////////////////////////////////////////
- (void) importSheetDidEnd: (NSOpenPanel *)panel
				returnCode: (int)returnCode
			   contextInfo: (void *)contextInfo;

////////////////////////////////////////////////////////////////////////////////
/// Updates the importProgressBar on the importStatusPanel based on the
/// current percent completion of the import task as reported by the reader.
////////////////////////////////////////////////////////////////////////////////
- (void) updateProgressBar;

- (void) importSpecialObjects;

- (NSManagedObject*) duplicateOfObject: (NSManagedObject*) object
							  isOfType: (int*) duplicateType;
							  
- (void) setAccountForImportingTransaction: (NSManagedObject*) object;

- (void) displayConfirmationTransactionInfo;

- (void) confirmTransactions;

- (void) populateAccountPopUpList;

@end
