////////////////////////////////////////////////////////////////////////////////
//  QIFReader.h
////////////////////////////////////////////////////////////////////////////////
///
/// @brief	Imports QIF files into the Import data model.
///
/// A QIFReader object is capable of reading in a QIF file and parsing it into
/// QIF objects in the data model.  It is also capable of translating 
/// QIF objects in the data model into Prospects objects in the data model.
///
/// Copyright (c) 2006 MotiMotion.
///
/// @author	Michael Rutt
///			<a href="mailto:michael@motimotion.com">michael@motimotion.com</a>
///			<a href="http://www.mikerutt.com">http://www.mikerutt.com</a>
///
////////////////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>
#import "BaseFileReader.h"


@interface QIFReader : BaseFileReader 
{
    /// Whether or not the QIF reader should ignore transfers when loading data.
	/// By default transfers are not ignored.
    BOOL					ignoreTransfers;
	
	/// Set to true if the transactions are tied to accounts in the QIF file
	BOOL					transactionsHaveAccounts;
	
	/// The QIFDocument imported
	NSManagedObject*		_document;
}

////////////////////////////////////////////////////////////////////////////////
/// Initializes the QIFReader with settings necessary to properly import the
/// data from the .qif file.
///
/// @param	coordinator		NSPersistenStoreCoordinator to use for the tempoary
///							SQLLite file that stores the QIF data.
/// @param	objectIDArray	Array to place the object IDs of imported data into
///							so another thread can process them.
/// @param	rules			The import rules to apply when importing (nil = no
///							rules to apply).
/// @param	target			The target to call the selector on for updating the
///							import progress.
/// @param	selector		The selector to call on the target for updating the
///							import progress.
////////////////////////////////////////////////////////////////////////////////
- (id) initWithContext: (NSManagedObjectContext*) context
  progressCallbackToTarget: (id) target
				  selector: (SEL) selector;

////////////////////////////////////////////////////////////////////////////////
/// Sets whether or not the import should ignore transfers.
////////////////////////////////////////////////////////////////////////////////
- (void) setIgnoreTransfers: (BOOL)ignore;

////////////////////////////////////////////////////////////////////////////////
/// Returns whether or not the importer is ignoring transfers.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) ignoreTransfers;

////////////////////////////////////////////////////////////////////////////////
/// Loads a QIF file from a document into the data model.  This function opens
/// the file, retrieves the data as a string, and passes the data on to
/// loadFromString:.  Returns true on success, false on error.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) loadFromFile: (NSString *)fileName;
				
////////////////////////////////////////////////////////////////////////////////
/// Loads a QIF file from the specified string into the data model.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) loadFromString: (NSString *)qifString;
				
////////////////////////////////////////////////////////////////////////////////
/// Cancels an import in progress.
////////////////////////////////////////////////////////////////////////////////
- (void) cancelImport;

////////////////////////////////////////////////////////////////////////////////
/// Checks imported accounts to see if any were already imported.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) wasAccountImportedWithName: (NSString*) name;

				
@end
