////////////////////////////////////////////////////////////////////////////////
//  ViewPointComponentController.h
////////////////////////////////////////////////////////////////////////////////
///
/// @brief	Base class for a controller of a component of a viewpoint instance.
///
/// This is a base class for a controller of a component of a viewpoint
/// instance.  It contains code and methods common amongst all components.
///
/// Copyright (c) 2006, 2007 MotiMotion.
///
/// @author	Michael Rutt
///			<a href="mailto:michael@motimotion.com">michael@motimotion.com</a>
///			<a href="http://www.mikerutt.com">http://www.mikerutt.com</a>
///
////////////////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>

/// View tag for a control selecting the timeframe for the viewpoint
#define		VP_TIMEFRAME_TAG	1
/// View tag for a control selecting the debit type (debit/deposit) for the vp
#define		VP_DEBIT_TYPE_TAG	2


@interface ViewPointComponentController : NSArrayController
{
	/// Outlet to the chart on the component view
	IBOutlet id			chart;

	/// The view displaying the data for this controller
	IBOutlet id			componentView;

	/// The accounts controller
	IBOutlet id			accountsController;
	
	/// The transactions controller that can be observed for selection changes
	IBOutlet id			transactionsController;
	
	/// Is the view of this component currently active?
	BOOL				active;
	
	/// The account being observed
	NSManagedObject*	observingAccount;

	/// The NSSlider timeframe value (if used)
	int					timeframe;
}


//==============================================================================
// Actions.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// If this viewpoint controller supports filtering the viewpoint data on
/// a timeframe adjusted by an NSSlider, this should be hooked up to it.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) timeframeChanged: (id) sender;


//==============================================================================
// Class methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Called by the ViewPointController whenever the select account changes.
////////////////////////////////////////////////////////////////////////////////
- (void) changedToAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// Sets this viewpoint component as active / inactive.  When inactive, the
/// viewpoint should perform as little processing as possible.
////////////////////////////////////////////////////////////////////////////////
- (void) setActive: (BOOL) value;

////////////////////////////////////////////////////////////////////////////////
/// Returns yes if the component is currently visible.
////////////////////////////////////////////////////////////////////////////////
- (BOOL) isActive;

////////////////////////////////////////////////////////////////////////////////
/// Called when this controller is made active; default does nothing.
////////////////////////////////////////////////////////////////////////////////
- (void) viewMadeActive;

////////////////////////////////////////////////////////////////////////////////
/// Called when the timeframe is changed; default does nothing.
////////////////////////////////////////////////////////////////////////////////
- (void) timeframeChanged;

////////////////////////////////////////////////////////////////////////////////
/// Returns the component view (viewpoint view) for this controller.
////////////////////////////////////////////////////////////////////////////////
- (NSView*) componentView;


//==============================================================================
// Helper methods.
// (These are available for subclasses to use as needed.  They are not used by
// this base class.)
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Registers the controller as an observer for the category, debit and deposit
/// values of all the transactions in the currently selected account.
////////////////////////////////////////////////////////////////////////////////
- (void) registerAsObserverOfTransactionsInSet: (NSMutableSet*) transactions;

////////////////////////////////////////////////////////////////////////////////
/// Deregisters the controller as an observer for the category, debit and
/// deposit values of all the transactions in the currently selected account.
////////////////////////////////////////////////////////////////////////////////
- (void) unregisterAsObserverOfTransactionsInSet: (NSMutableSet*) transactions;


//==============================================================================
// Abstract class methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Must be overridden by the base class to perform necessary deregistrations
/// for the specified account.  Called when an account is no longer visible.
////////////////////////////////////////////////////////////////////////////////
- (void) unregisterObservationsForAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// Must be overridden by the base class to perform necessary registrations
/// for the specified account.  Called when an account is selected.
////////////////////////////////////////////////////////////////////////////////
- (void) registerObservationsForAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// Should be overridden by the base class to perform necessary viewpoint
/// changes after a new account has been selected.  Should retrieve settings
/// and modify viewpoint per the settings.
////////////////////////////////////////////////////////////////////////////////
- (void) updateViewPointForAccount: (NSManagedObject*) account;

@end
