////////////////////////////////////////////////////////////////////////////////
//  ViewPointController.h
////////////////////////////////////////////////////////////////////////////////
///
/// @brief	Controls a viewpoint instance.
///
/// This class controls a viewpoint instance.  It is responsible for managing
/// the account selection changing, as well as setting up all the viewpoint
/// controllers and views as necessary.
///
/// It notifies viewpoint controllers when the following changes occur:
///  * The account being viewed
///  * The selected transaction
///  * Deleted or added transaction
///  * Change to a transaction category
///  * Change to a transaction amount
///  * Change to a transaction date
///  * Change to an account balance
///
/// Copyright (c) 2006, 2007 MotiMotion.
///
/// @author	Michael Rutt
///			<a href="mailto:michael@motimotion.com">michael@motimotion.com</a>
///			<a href="http://www.mikerutt.com">http://www.mikerutt.com</a>
///
////////////////////////////////////////////////////////////////////////////////

#import <Cocoa/Cocoa.h>
#import <MotiFoundation/MotiViewPointController.h>
#import "CategoryViewPointController.h"


////////////////////////////////////////////////////////////////////////////////
/// This class controls a viewpoint instance.  It is responsible for registering
/// for all the necessary events and distributing them to the viewpoints.
////////////////////////////////////////////////////////////////////////////////
@interface ViewPointController : MotiViewPointController
{
	/// The viewpoints manager
	IBOutlet id						vpManager;

	/// The accounts controller
	IBOutlet id						accountsController;
	
	/// The transactions controller
	IBOutlet id						transactionsController;
	
	/// The transactions controller
	IBOutlet id						vpTransactionsController;
	
	/// The splits controller - used for transaction selection
	IBOutlet id						splitsController;

	/// The calendar viewpoint controller
	IBOutlet id						calendarVPController;
	
	/// The recent transactions viewpoint controller
	IBOutlet id						recentTransactionsVPController;
	
	/// The balance history viewpoint controller
	IBOutlet id						balanceHistoryVPController;
	
	/// The category viewpoint controller
	IBOutlet id						categoryVPController;
	
	/// The category history viewpoint controller
	IBOutlet id						categoryHistoryVPController;
	
	/// The left-over viewpoint controller
	IBOutlet id						leftOverVPController;
	
	/// The splitview delegate managing this viewpoint's splitview (if any)
	IBOutlet id						splitViewDelegate;
	
	//==========================================================================

	/// A convenience array of controllers so we can loop through them instead
	/// of having to explicity call each one when things occur; this makes it
	/// easier to add new controllers
	NSMutableArray*					controllers;
	
	/// Points to the currently active controller
	id								activeController;
	
	//==========================================================================
		
	/// The managed object context to use when working with the data
	NSManagedObjectContext*			managedObjectContext;
	
	/// The account currently being observed by this viewpoint
	NSManagedObject*				observingAccount;
	
	/// Set to YES when we're importing; this stops it from notifying all the
	/// viewpoint controllers to speed things up
	BOOL							importing;
	
	//==========================================================================

	/// NOTE: When both of these are set, a category has changed and we can
	///       notify the viewpoint controllers
	/// YES when the controller detected a transaction category changed
	BOOL							_controllerTransCategoryChanged;
	/// Array of data to send to viewpoints when a transaction category changes
	/// this is nil if nothing detected
	NSMutableArray*					_transCategoryChangedData;

	/// NOTE: When both of these are set, a transaction amount has changed and
	///		  we can notify the viewpoint controllers
	/// YES when the controller detected a transaction amount changed
	BOOL							_controllerTransAmountChanged;
	/// Array of data to send to viewpoints when a transaction amount changes
	/// this is nil if nothing detected
	NSMutableArray*					_transAmountChangedData;

	/// NOTE: When both of these are set, a transaction date has changed and
	///		  we can notify the viewpoint controllers
	/// YES when the controller detected a transaction date changed
	BOOL							_controllerTransDateChanged;
	/// Array of data to send to viewpoints when a transaction date changes
	/// this is nil if nothing detected
	NSMutableArray*					_transDateChangedData;
	
	/// When the list of transactions changes (add / delete)
	BOOL							_controllerTransListChanged;
	NSMutableArray*					_transListChangedData;
	/// YES if added transaction(s); NO if deleted
	BOOL							_controllerTransListAdded;
	
	NSString*						dataChangedSync;
}


//==============================================================================
// Class methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Called whenever the account selection has changed (via KVC observing).
////////////////////////////////////////////////////////////////////////////////
- (void) accountChanged;

- (void) registerControllerObservations: (id) object;

////////////////////////////////////////////////////////////////////////////////
/// Returns the component controller for the specified viewpoint view.
////////////////////////////////////////////////////////////////////////////////
- (ViewPointComponentController*) getControllerForViewPoint: (NSView*) viewpoint;

////////////////////////////////////////////////////////////////////////////////
/// Called after a viewpoint animation has ended.  The viewpoint is made active.
////////////////////////////////////////////////////////////////////////////////
- (void) viewPointAnimationEnded;

////////////////////////////////////////////////////////////////////////////////
/// Called whenever the ImportManager posts aan import starting notification.
/// Causes active viewpoints to become inactive so the import does not come to
/// a screeching halt.
////////////////////////////////////////////////////////////////////////////////
- (void) importStarted: (NSNotification*) notification;

////////////////////////////////////////////////////////////////////////////////
/// Called whenever the ImportManager posts aan import ending notification.
/// Causes the viewpoints that were active to become active again.
////////////////////////////////////////////////////////////////////////////////
- (void) importEnded: (NSNotification*) notification;

////////////////////////////////////////////////////////////////////////////////
/// TODO
////////////////////////////////////////////////////////////////////////////////
- (void) registerObservationsForAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// TODO
////////////////////////////////////////////////////////////////////////////////
- (void) unregisterObservationsForAccount: (NSManagedObject*) account;

////////////////////////////////////////////////////////////////////////////////
/// TODO
////////////////////////////////////////////////////////////////////////////////
- (void) registerObservationsForTransactions: (NSArray*) transactions;

////////////////////////////////////////////////////////////////////////////////
/// TODO
////////////////////////////////////////////////////////////////////////////////
- (void) unregisterObservationsForTransactions: (NSArray*) transactions;


//==============================================================================
// Notification methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// TODO
////////////////////////////////////////////////////////////////////////////////
- (void) notifyForTransactionChange: (NSMutableArray*) dataArray
				  controllerChanged: (BOOL*) changed;

////////////////////////////////////////////////////////////////////////////////
/// TODO
////////////////////////////////////////////////////////////////////////////////
- (void) notifyForTransactionListChange: (NSMutableArray*) transactions
					  controllerChanged: (BOOL*) changed
								  added: (BOOL) added;


//==============================================================================
// Overridden methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// Overrides base class and stores visible viewpoint in the settings.  Also
/// controls the active state of the viewpoints affected by the change.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) nextViewPoint: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Overrides base class and stores visible viewpoint in the settings.  Also
/// controls the active state of the viewpoints affected by the change.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) previousViewPoint: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Overrides base class and stores visible viewpoint in the settings.  Also
/// controls the active state of the viewpoints affected by the change.
////////////////////////////////////////////////////////////////////////////////
- (IBAction) changeViewPoint: (id) sender;

////////////////////////////////////////////////////////////////////////////////
/// Overrides base class and controls the active state of the viewpoints
/// affected by the change.
////////////////////////////////////////////////////////////////////////////////
- (void) changeToViewPointWithName: (NSString*) name;


//==============================================================================
// NSKeyValueObserving protocol methods.
//==============================================================================


////////////////////////////////////////////////////////////////////////////////
/// This message is called when the value at the specified key path relative to
/// the given object has changed.  Specifically, this is used to monitor the
/// account selection changing via the accountsController.
////////////////////////////////////////////////////////////////////////////////
- (void) observeValueForKeyPath:(NSString *)keyPath
					   ofObject:(id)object 
						 change:(NSDictionary *)change
						context:(void *)context;

@end
